-----------------------------------------------------------------------------------------------------------
--DDL ST_Geometry Table
-----------------------------------------------------------------------------------------------------------

CREATE TABLE ST_DEMO
  (
    FID NUMBER NOT NULL,
    SHAPE SDE.ST_GEOMETRY,
    CONSTRAINT FID_UK UNIQUE (FID)
  );
  
CREATE INDEX SHP_IDX ON ST_DEMO
  (
    SHAPE
  )
  INDEXTYPE IS SDE.ST_SPATIAL_INDEX PARAMETERS('ST_GRIDS=0 ST_SRID=4326');
  
  
CREATE SEQUENCE ST_DEMO_SQ INCREMENT BY 1;
  
  
-----------------------------------------------------------------------------------------------------------  
--DML ST_Geometry Table
-----------------------------------------------------------------------------------------------------------

--POINT

INSERT
INTO ST_DEMO VALUES
  (
    ST_DEMO_SQ.NEXTVAL,
    SDE.ST_GEOMETRY('POINT (-117.1625 32.7150)', 4326)
  );

--LINESTRING
  
INSERT
INTO ST_DEMO VALUES
  (
    ST_DEMO_SQ.NEXTVAL,
    SDE.ST_GEOMETRY('LINESTRING (-117.1625 32.7150, -117.1 32.7)', 4326)
  );
  
--POLYGON  
  
INSERT
INTO ST_DEMO VALUES
  (
    ST_DEMO_SQ.NEXTVAL,
    SDE.ST_GEOMETRY('POLYGON ((-117.12 32.7, -117.1 32.7, -117.1 32.6, -117.12 32.6, -117.12 32.7))', 4326)
  );
  
--Optimized POINT
  
INSERT
INTO ST_DEMO VALUES
  (
    ST_DEMO_SQ.NEXTVAL,
    SDE.ST_GEOMETRY(-117.1625, 32.7150, null, null, 4326)
  );
  

-----------------------------------------------------------------------------------------------------------  
--ST_Geometry: Accessors
-----------------------------------------------------------------------------------------------------------

--ST_SRID

SELECT DISTINCT SDE.ST_SRID(SHAPE) AS SRID FROM ST_POINTS;
SELECT DISTINCT SDE.ST_SRID(SHAPE) AS SRID FROM ST_LINES;
SELECT DISTINCT SDE.ST_SRID(SHAPE) AS SRID FROM ST_POLYGONS;

--COORDINATES:

--ST_IS3D
SELECT SDE.ST_IS3D(SHAPE) AS Z_DIM FROM ST_POINTS;

--ST_ISMEASURED
SELECT SDE.ST_ISMEASURED(SHAPE) AS M_DIM FROM ST_POINTS;

--X/Y
SELECT SDE.ST_X(SHAPE) AS X, SDE.ST_Y(SHAPE) AS Y FROM ST_POINTS;

SELECT  SDE.ST_X(SDE.ST_STARTPOINT(SHAPE)) AS STARTX, 
		SDE.ST_Y(SDE.ST_STARTPOINT(SHAPE)) AS STARTY, 
		SDE.ST_X(SDE.ST_ENDPOINT(SHAPE)) AS ENDX, 
		SDE.ST_Y(SDE.ST_ENDPOINT(SHAPE)) AS ENDY		
		FROM ST_LINES;
		
--TEXT/BINARY
SELECT SDE.ST_ASTEXT(SHAPE) AS WKT FROM ST_POINTS; 
SELECT SDE.ST_ASBINARY(SHAPE) AS WKB FROM ST_POINTS;

--ENVELOPE
SELECT SDE.ST_ENVELOPE(SHAPE) AS ENV FROM ST_LINES;


-----------------------------------------------------------------------------------------------------------  
--ST_Geometry: Relationships/Operations
-----------------------------------------------------------------------------------------------------------

--ST_TRANSFORM
SELECT SDE.ST_ASTEXT(SDE.ST_TRANSFORM(SHAPE, 2229)) AS WKT FROM ST_POINTS;

SELECT SDE.ST_ASTEXT(SDE.ST_TRANSFORM(SHAPE, 2229)) AS WKT FROM ST_POLYGONS;

SELECT  SDE.ST_LENGTH(SDE.ST_TRANSFORM(SHAPE, 4269)) AS NAD83_dd,
		SDE.ST_LENGTH(SDE.ST_TRANSFORM(SHAPE, 5070)) AS ALBERS_CONUS_m,
		SDE.ST_LENGTH(SHAPE) AS CAL5_ft
		FROM ST_LINES;
		

--ST_CONTAINS/ST_WITHIN

SELECT PNT.OBJECTID
FROM ST_POLYGONS POLY,
  ST_POINTS PNT
WHERE SDE.ST_CONTAINS(POLY.SHAPE, PNT.SHAPE) <> 0;

SELECT PNT.OBJECTID
FROM ST_POLYGONS POLY,
  ST_POINTS PNT
WHERE SDE.ST_WITHIN(PNT.SHAPE, POLY.SHAPE) <> 0;

SELECT PNT.OBJECTID
FROM ST_POLYGONS POLY,
  ST_POINTS PNT
WHERE SDE.ST_CONTAINS(POLY.SHAPE, PNT.SHAPE) <> 0
AND SDE.ST_WITHIN(PNT.SHAPE, POLY.SHAPE) <> 0;

--ST_INTERSECTS

SELECT distinct PNT.OBJECTID
FROM ST_POLYGONS POLY,
  ST_POINTS PNT
WHERE SDE.ST_INTERSECTS(POLY.SHAPE, PNT.SHAPE) <> 0;

--ST_BUFFER

SELECT SDE.ST_BUFFER(SDE.ST_TRANSFORM(SHAPE, 2229), 50) AS SHAPE, CAST(ROWNUM AS VARCHAR(10)) AS FID FROM ST_POINTS
UNION
SELECT SDE.ST_BUFFER(SHAPE, 25) AS SHAPE, CAST(ROWNUM AS VARCHAR(10)) AS FID FROM ST_LINES;

--NESTED FUNCTIONS

SELECT distinct PNT.OBJECTID
FROM ST_LINES LINE,
  ST_POINTS PNT
WHERE SDE.ST_INTERSECTS(SDE.ST_TRANSFORM(SDE.ST_BUFFER(LINE.SHAPE, 1), 4269), PNT.SHAPE) <> 0;

SELECT PNT.OBJECTID
FROM ST_LINES LINE,
  ST_POINTS PNT
WHERE SDE.ST_CONTAINS(SDE.ST_BUFFER(LINE.SHAPE, 1), SDE.ST_TRANSFORM(PNT.SHAPE, 2229)) <> 0;

--MORE ST_FUNCTIONS

CREATE TABLE ST_BUFFER_DATA AS 
SELECT SDE.ST_BUFFER(SDE.ST_TRANSFORM(SHAPE, 2229), 50) AS SHAPE, CAST(ROWNUM AS VARCHAR(10)) AS FID FROM ST_POINTS
UNION
SELECT SDE.ST_BUFFER(SHAPE, 25) AS SHAPE, CAST(ROWNUM AS VARCHAR(10)) AS FID FROM ST_LINES;

SELECT SDE.ST_GEOMFROMWKB(SDE.ST_ASBINARY(SDE.ST_AGGR_UNION(A.SHAPE)), 2229) AS SHAPE, '1' FROM 
(SELECT SDE.ST_BUFFER(SDE.ST_TRANSFORM(SHAPE, 2229), 50) AS SHAPE, CAST(ROWNUM AS VARCHAR(10)) AS FID FROM ST_POINTS
UNION
SELECT SDE.ST_BUFFER(SHAPE, 25) AS SHAPE, CAST(ROWNUM AS VARCHAR(10)) AS FID FROM ST_LINES) A;
